# Generated from AtopileParser.g4 by ANTLR 4.13.0
from antlr4 import *
if "." in __name__:
    from .AtopileParser import AtopileParser
else:
    from AtopileParser import AtopileParser

# This class defines a complete listener for a parse tree produced by AtopileParser.
class AtopileParserListener(ParseTreeListener):

    # Enter a parse tree produced by AtopileParser#file_input.
    def enterFile_input(self, ctx:AtopileParser.File_inputContext):
        pass

    # Exit a parse tree produced by AtopileParser#file_input.
    def exitFile_input(self, ctx:AtopileParser.File_inputContext):
        pass


    # Enter a parse tree produced by AtopileParser#stmt.
    def enterStmt(self, ctx:AtopileParser.StmtContext):
        pass

    # Exit a parse tree produced by AtopileParser#stmt.
    def exitStmt(self, ctx:AtopileParser.StmtContext):
        pass


    # Enter a parse tree produced by AtopileParser#simple_stmts.
    def enterSimple_stmts(self, ctx:AtopileParser.Simple_stmtsContext):
        pass

    # Exit a parse tree produced by AtopileParser#simple_stmts.
    def exitSimple_stmts(self, ctx:AtopileParser.Simple_stmtsContext):
        pass


    # Enter a parse tree produced by AtopileParser#simple_stmt.
    def enterSimple_stmt(self, ctx:AtopileParser.Simple_stmtContext):
        pass

    # Exit a parse tree produced by AtopileParser#simple_stmt.
    def exitSimple_stmt(self, ctx:AtopileParser.Simple_stmtContext):
        pass


    # Enter a parse tree produced by AtopileParser#compound_stmt.
    def enterCompound_stmt(self, ctx:AtopileParser.Compound_stmtContext):
        pass

    # Exit a parse tree produced by AtopileParser#compound_stmt.
    def exitCompound_stmt(self, ctx:AtopileParser.Compound_stmtContext):
        pass


    # Enter a parse tree produced by AtopileParser#block.
    def enterBlock(self, ctx:AtopileParser.BlockContext):
        pass

    # Exit a parse tree produced by AtopileParser#block.
    def exitBlock(self, ctx:AtopileParser.BlockContext):
        pass


    # Enter a parse tree produced by AtopileParser#blockdef.
    def enterBlockdef(self, ctx:AtopileParser.BlockdefContext):
        pass

    # Exit a parse tree produced by AtopileParser#blockdef.
    def exitBlockdef(self, ctx:AtopileParser.BlockdefContext):
        pass


    # Enter a parse tree produced by AtopileParser#blocktype.
    def enterBlocktype(self, ctx:AtopileParser.BlocktypeContext):
        pass

    # Exit a parse tree produced by AtopileParser#blocktype.
    def exitBlocktype(self, ctx:AtopileParser.BlocktypeContext):
        pass


    # Enter a parse tree produced by AtopileParser#import_stmt.
    def enterImport_stmt(self, ctx:AtopileParser.Import_stmtContext):
        pass

    # Exit a parse tree produced by AtopileParser#import_stmt.
    def exitImport_stmt(self, ctx:AtopileParser.Import_stmtContext):
        pass


    # Enter a parse tree produced by AtopileParser#assign_stmt.
    def enterAssign_stmt(self, ctx:AtopileParser.Assign_stmtContext):
        pass

    # Exit a parse tree produced by AtopileParser#assign_stmt.
    def exitAssign_stmt(self, ctx:AtopileParser.Assign_stmtContext):
        pass


    # Enter a parse tree produced by AtopileParser#assignable.
    def enterAssignable(self, ctx:AtopileParser.AssignableContext):
        pass

    # Exit a parse tree produced by AtopileParser#assignable.
    def exitAssignable(self, ctx:AtopileParser.AssignableContext):
        pass


    # Enter a parse tree produced by AtopileParser#retype_stmt.
    def enterRetype_stmt(self, ctx:AtopileParser.Retype_stmtContext):
        pass

    # Exit a parse tree produced by AtopileParser#retype_stmt.
    def exitRetype_stmt(self, ctx:AtopileParser.Retype_stmtContext):
        pass


    # Enter a parse tree produced by AtopileParser#connect_stmt.
    def enterConnect_stmt(self, ctx:AtopileParser.Connect_stmtContext):
        pass

    # Exit a parse tree produced by AtopileParser#connect_stmt.
    def exitConnect_stmt(self, ctx:AtopileParser.Connect_stmtContext):
        pass


    # Enter a parse tree produced by AtopileParser#connectable.
    def enterConnectable(self, ctx:AtopileParser.ConnectableContext):
        pass

    # Exit a parse tree produced by AtopileParser#connectable.
    def exitConnectable(self, ctx:AtopileParser.ConnectableContext):
        pass


    # Enter a parse tree produced by AtopileParser#signaldef_stmt.
    def enterSignaldef_stmt(self, ctx:AtopileParser.Signaldef_stmtContext):
        pass

    # Exit a parse tree produced by AtopileParser#signaldef_stmt.
    def exitSignaldef_stmt(self, ctx:AtopileParser.Signaldef_stmtContext):
        pass


    # Enter a parse tree produced by AtopileParser#pindef_stmt.
    def enterPindef_stmt(self, ctx:AtopileParser.Pindef_stmtContext):
        pass

    # Exit a parse tree produced by AtopileParser#pindef_stmt.
    def exitPindef_stmt(self, ctx:AtopileParser.Pindef_stmtContext):
        pass


    # Enter a parse tree produced by AtopileParser#with_stmt.
    def enterWith_stmt(self, ctx:AtopileParser.With_stmtContext):
        pass

    # Exit a parse tree produced by AtopileParser#with_stmt.
    def exitWith_stmt(self, ctx:AtopileParser.With_stmtContext):
        pass


    # Enter a parse tree produced by AtopileParser#new_stmt.
    def enterNew_stmt(self, ctx:AtopileParser.New_stmtContext):
        pass

    # Exit a parse tree produced by AtopileParser#new_stmt.
    def exitNew_stmt(self, ctx:AtopileParser.New_stmtContext):
        pass


    # Enter a parse tree produced by AtopileParser#name_or_attr.
    def enterName_or_attr(self, ctx:AtopileParser.Name_or_attrContext):
        pass

    # Exit a parse tree produced by AtopileParser#name_or_attr.
    def exitName_or_attr(self, ctx:AtopileParser.Name_or_attrContext):
        pass


    # Enter a parse tree produced by AtopileParser#numerical_pin_ref.
    def enterNumerical_pin_ref(self, ctx:AtopileParser.Numerical_pin_refContext):
        pass

    # Exit a parse tree produced by AtopileParser#numerical_pin_ref.
    def exitNumerical_pin_ref(self, ctx:AtopileParser.Numerical_pin_refContext):
        pass


    # Enter a parse tree produced by AtopileParser#attr.
    def enterAttr(self, ctx:AtopileParser.AttrContext):
        pass

    # Exit a parse tree produced by AtopileParser#attr.
    def exitAttr(self, ctx:AtopileParser.AttrContext):
        pass


    # Enter a parse tree produced by AtopileParser#totally_an_integer.
    def enterTotally_an_integer(self, ctx:AtopileParser.Totally_an_integerContext):
        pass

    # Exit a parse tree produced by AtopileParser#totally_an_integer.
    def exitTotally_an_integer(self, ctx:AtopileParser.Totally_an_integerContext):
        pass


    # Enter a parse tree produced by AtopileParser#name.
    def enterName(self, ctx:AtopileParser.NameContext):
        pass

    # Exit a parse tree produced by AtopileParser#name.
    def exitName(self, ctx:AtopileParser.NameContext):
        pass


    # Enter a parse tree produced by AtopileParser#string.
    def enterString(self, ctx:AtopileParser.StringContext):
        pass

    # Exit a parse tree produced by AtopileParser#string.
    def exitString(self, ctx:AtopileParser.StringContext):
        pass


    # Enter a parse tree produced by AtopileParser#boolean_.
    def enterBoolean_(self, ctx:AtopileParser.Boolean_Context):
        pass

    # Exit a parse tree produced by AtopileParser#boolean_.
    def exitBoolean_(self, ctx:AtopileParser.Boolean_Context):
        pass



del AtopileParser